// ________________________________________________________________________________________________________
// Copyright (c) 2020 GebraBit Inc. All rights reserved.
//
// This software, related documentation and any modifications thereto (collectively �Software�) is subject
// to GebraBit and its licensors' intellectual property rights under U.S. and international copyright
// and other intellectual property rights laws. 
//
// GebraBit and its licensors retain all intellectual property and proprietary rights in and to the Software
// and any use, reproduction, disclosure or distribution of the Software without an express license agreement
// from GebraBit is strictly prohibited.
 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT 
// NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT IN  
// NO EVENT SHALL GebraBit BE LIABLE FOR ANY DIRECT, SPECIAL, INDIRECT, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, 
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
// NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
// OF THE SOFTWARE.
// @Author       	: Sepehr Azimi
// inspired by ADS1292R ProtoCentral source code
// ________________________________________________________________________________________________________

#ifndef ads119X_h
#define ads119X_h

#include "Arduino.h"

#define CONFIG_SPI_MASTER_DUMMY   0xFF

// Register Read Commands
#define RREG  0x20;		//Read n nnnn registers starting at address r rrrr
                      //first byte 001r rrrr (2xh)(2) - second byte 000n nnnn(2)
#define WREG  0x40;	  //Write n nnnn registers starting at address r rrrr
                      //first byte 010r rrrr (2xh)(2) - second byte 000n nnnn(2)
#define START	0x08		//Start/restart (synchronize) conversions
#define STOP	0x0A		//Stop conversion
#define RDATAC 0x10		//Enable Read Data Continuous mode.

//This mode is the default mode at power-up.
#define SDATAC 0x11		//Stop Read Data Continuously mode
#define RDATA	0x12		//Read data by command; supports multiple read back.

//register address
#define ADS119X_REG_ID			  0x00
#define ADS119X_REG_CONFIG1		0x01
#define ADS119X_REG_CONFIG2		0x02
#define ADS119X_REG_LOFF		  0x03
#define ADS119X_REG_CH1SET		0x04
#define ADS119X_REG_CH2SET		0x05
#define ADS119X_REG_RLDSENS		0x06
#define ADS119X_REG_LOFFSENS  0x07
#define ADS119X_REG_LOFFSTAT  0x08
#define ADS119X_REG_MISC1	    0x09
#define ADS119X_REG_MISC2	    0x0A

//Packet format
#define  CES_CMDIF_PKT_START_1   0x0A
#define  CES_CMDIF_PKT_START_2   0xFA
#define  CES_CMDIF_TYPE_DATA     0x02
#define  CES_CMDIF_PKT_STOP_1    0x00
#define  CES_CMDIF_PKT_STOP_2    0x0B

typedef struct Record{
  volatile signed long sDaqVals[8];
  boolean leadoffDetected = true;
  signed long sresultTempResp;
}ads119xOutputValues;

class ads119x
{
  public:
    boolean getads119xEcgSamples(const int dataReady,const int chipSelect,ads119xOutputValues *ecgRespirationValues);
    static void ads119xInit(const int chipSelect,const int pwdnPin,const int startPin);
    static void ads119xReset(const int pwdnPin);

  private:
	  static void ads119xRegWrite (unsigned char READ_WRITE_ADDRESS, unsigned char DATA,const int chipSelect);
    static void ads119xSPICommandData(unsigned char dataIn,const int chipSelect);
    static void ads119xDisableStart(const int startPin);
    static void ads119xEnableStart(const int startPin);
    static void ads119xHardStop (const int startPin);
    static void ads119xStartDataConvCommand (const int chipSelect);
    static void ads119xSoftStop (const int chipSelect);
    static void ads119xStartReadDataContinuous (const int chipSelect);
    static void ads119xStopReadDataContinuous (const int chipSelect);
    static char* ads119xReadData(const int chipSelect);
};

#endif
